;; Generated package description from ntlm.el  -*- no-byte-compile: t -*-
(define-package "ntlm" "2.1.0" "NTLM (NT LanManager) authentication support" 'nil :commit "1add5ec3a270cba99549dc7e78c529c3dd190784" :url "https://elpa.gnu.org/packages/ntlm.html" :authors '(("Taro Kawagishi" . "tarok@transpulse.org")) :maintainer '("Thomas Fitzsimmons" . "fitzsim@fitzsim.org") :keywords '("ntlm" "sasl" "comm"))
